package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;
import de.intevation.flys.utils.KMIndex;

/**
 * Facet to show average W values for Q sectors.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixLongitudinalAnalysisFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixLongitudinalAnalysisFacet.class);

    /** Trivial Constructor. */
    public FixLongitudinalAnalysisFacet() {
    }


    public FixLongitudinalAnalysisFacet(
        int ndx,
        String name,
        String description)
    {
        super(
            ndx,
            name,
            description,
            ComputeType.ADVANCE,
            null,
            null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext.
     *
     * @return the data as KMIndex.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixLongitudinalAnalysisFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();

            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();
            if (kmPeriods == null) {
                logger.warn("No analysis periods found.");
                return null;
            }
            int periodNdx = index >> 8;
            int qwdNdx = index & 255;
            KMIndex<QWD> resPeriods =
                    new KMIndex<QWD>();
            for (KMIndex.Entry<AnalysisPeriod[]> entry: kmPeriods) {
                AnalysisPeriod ap = entry.getValue()[periodNdx];
                QWD[] qwds = ap.qwds;
                for(int i = 0; i < qwds.length; i++) {
                    if(qwds[i].getIndex() == qwdNdx) {
                        resPeriods.add(entry.getKm(), qwds[i]);
                    }
                }
            }


            return resPeriods;
        }
        else {
            logger.warn("Artifact is no instance of FLYSArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixLongitudinalAnalysisFacet deepCopy() {
        FixLongitudinalAnalysisFacet copy = new FixLongitudinalAnalysisFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
